=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2008-04-03 19:37:54 +0000
+++ bzrlib/bzrdir.py	2008-04-03 19:39:27 +0000
@@ -356,14 +356,17 @@
         bzrdir._find_or_create_repository(force_new_repo)
         return bzrdir.create_branch()
 
+    def determine_repository_policy(self, force_new_repo=False):
+        if not force_new_repo:
+            try:
+                return UseExistingRepository(self.find_repository())
+            except errors.NoRepositoryPresent:
+                pass
+        return CreateRepository(self)
+
     def _find_or_create_repository(self, force_new_repo):
         """Create a new repository if needed, returning the repository."""
-        if force_new_repo:
-            return self.create_repository()
-        try:
-            return self.find_repository()
-        except errors.NoRepositoryPresent:
-            return self.create_repository()
+        return self.determine_repository_policy(force_new_repo).apply()
         
     @staticmethod
     def create_branch_convenience(base, force_new_repo=False,
@@ -2615,6 +2618,24 @@
         return output
 
 
+class CreateRepository(object):
+
+    def __init__(self, bzrdir):
+        self._bzrdir = bzrdir
+
+    def apply(self):
+        return self._bzrdir.create_repository()
+
+
+class UseExistingRepository(object):
+
+    def __init__(self, repository):
+        self._repository = repository
+
+    def apply(self):
+        return self._repository
+
+
 format_registry = BzrDirFormatRegistry()
 format_registry.register('weave', BzrDirFormat6,
     'Pre-0.8 format.  Slower than knit and does not'

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2008-02-13 22:11:40 +0000
+++ bzrlib/tests/test_bzrdir.py	2008-04-03 19:39:27 +0000
@@ -453,6 +453,12 @@
         branch.bzrdir.open_repository()
         branch.bzrdir.open_workingtree()
 
+    def test_apply_repository_policy_standalone(self):
+        my_bzrdir = self.make_bzrdir('.')
+        repo_policy = my_bzrdir.determine_repository_policy()
+        repo = repo_policy.apply()
+        self.assertEqual(repo.bzrdir.root_transport.base,
+                         my_bzrdir.root_transport.base)
 
 class ChrootedTests(TestCaseWithTransport):
     """A support class that provides readonly urls outside the local namespace.

